package com.aliyun.emas.flutter.crashlytics;

import android.content.Context;

import androidx.annotation.NonNull;

import com.alibaba.ha.bizerrorreporter.BizErrorReporter;
import com.alibaba.ha.bizerrorreporter.module.AggregationType;
import com.alibaba.ha.bizerrorreporter.module.BizErrorModule;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** EmasCrashlyticsPlugin */
public class EmasCrashlyticsPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context mContext;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "emas_crashlytics");
    channel.setMethodCallHandler(this);
    mContext = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("captureException")) {
      int errorType = 12;
      String businessType = "HA_FLUTTER_ERROR";

      try {
        String error = call.argument("error");
        String stackTrace = call.argument("stackTrace");
        String routeName = call.argument("routeName");
        Map<String, String> message = call.argument("message");
        StringBuilder sb = new StringBuilder().append(error).append("\n\r").append(stackTrace);
        sendBizError(errorType, businessType, error, sb.toString(), routeName, message);
        result.success("0");
      } catch (ClassCastException e) {
        result.error("-1", e.getMessage(), e);
      } catch (Exception e) {
        result.error("-1", e.getMessage(), e);
      }
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    mContext = null;
  }

  private void sendBizError(int errorType, String businessType, String error, String stackTrace, String routeName, Map<String, String> message) {
    BizErrorModule bizErrorModule = new BizErrorModule();
    bizErrorModule.businessType = businessType;
    bizErrorModule.aggregationType = AggregationType.CONTENT;
    bizErrorModule.exceptionId = bizErrorModule.businessType + "_" + System.currentTimeMillis();
    bizErrorModule.exceptionCode = error;
    bizErrorModule.exceptionDetail = stackTrace;
    bizErrorModule.thread = null;
    bizErrorModule.exceptionVersion = "1.0.0.0";
    bizErrorModule.exceptionArg1 = routeName;
    bizErrorModule.exceptionArg2 = "arg2";
    bizErrorModule.exceptionArg3 = "arg3";
    bizErrorModule.errorType = errorType;
    bizErrorModule.crossPlatformCrashInfo = message;

    BizErrorReporter.getInstance().send(mContext, bizErrorModule);
  }
}
