#import "EmasCrashlyticsPlugin.h"
#import <BizErrorReporter4iOS/BizErrorReporter.h>
#import <BizErrorReporter4iOS/AdapterExceptionModule.h>

@implementation EmasCrashlyticsPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"emas_crashlytics"
            binaryMessenger:[registrar messenger]];
  EmasCrashlyticsPlugin* instance = [[EmasCrashlyticsPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([call.method isEqualToString:@"captureException"]) {
      NSString *businessType = @"HA_FLUTTER_ERROR";
      NSString *error = [call.arguments valueForKey:@"error"];
      NSString *stackTrace = [call.arguments valueForKey:@"stackTrace"];
      NSString *routeName = [call.arguments valueForKey:@"routeName"];
      NSDictionary *messageDict = [call.arguments valueForKey:@"message"];
      [self reportException:businessType summary:routeName detail:stackTrace error:error messageDict:messageDict];
      result(@"0");
  } else {
      result(FlutterMethodNotImplemented);
  }

}

/**
 * report exception through rest api
 */
- (void)reportException:(NSString *)exceptionType summary:(NSString *)summary detail:(NSString *)detail error:(NSString *)error messageDict:(NSDictionary *)messageDict {
    
    MotuReportAdapteHandler * handler = [[MotuReportAdapteHandler alloc] init];
    
    AdapterExceptionModule* exceptionModule = [[AdapterExceptionModule alloc] init];

    exceptionModule.customizeBusinessType = exceptionType;

    exceptionModule.aggregationType = ADAPTER_CONTENT;

    // 错误码，选择按内容聚合时，服务端根据错误码进行聚合，flutter routeName
    exceptionModule.exceptionCode = error;
    
    // 错误明细数据, flutter stack
    exceptionModule.exceptionDetail = detail;
    
    // 错误摘要, flutter error
    exceptionModule.exceptionArg1 = summary;
    
    // flutter message dict
    if (messageDict && [messageDict isKindOfClass:[NSDictionary class]]) {
        exceptionModule.exceptionArgs = @{@"message" : messageDict};
    }

    exceptionModule.exceptionVersion = @"1.0";
    
    //适配和上传错误数据
    [handler adapterWithExceptionModule:exceptionModule];
}


@end
