import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:emas_core/emas_core.dart';
export 'package:emas_core/emas_core.dart';

class EmasCrashlytics {
  /// 单例
  static EmasCrashlytics _instance;

  /// 引用
  static EmasCrashlytics get instance => _instance ?? EmasCrashlytics();

  /// 保存单例
  EmasCrashlytics() {
    _instance = this;
  }

  /// 异常上报管道方法
  MethodChannel _channel = const MethodChannel('emas_crashlytics');

  /// 上报异常信息
  Future<bool> captureException(
      {@required dynamic error,
      dynamic stackTrace,
      Map<String, String> message}) async {
    if (stackTrace == null && error != null && error.stackTrace != null) {
      stackTrace = error.stackTrace.toString();
    }
    bool result = await this._captureException(
        error: error.toString(),
        stackTrace: stackTrace.toString(),
        routeName: EmasCore.instance.config.routeName,
        message: message);
    return result;
  }

  /// 上报异常信息通道方法
  Future<bool> _captureException(
      {

      /// 错误简介
      String error,

      /// 错误堆栈
      String stackTrace,

      /// 页面路由信息
      String routeName,

      /// 自定义信息
      Map<String, String> message}) async {
    Map param = {
      "routeName": routeName ?? 'null',
      "error": error,
      "stackTrace": stackTrace,
    };
    if (message != null) {
      param['message'] = message;
    }
    var result;
    try {
      // 在上报成功时，目前 iOS 和 Android 返回值是 '0'
      result = await _channel.invokeMethod('captureException', param);
      result = result.toString();
      if (result == '0') {
        result = true;
      } else {
        result = false;
      }
    } catch (error) {
      result = false;
    }
    return result;
  }
}
